<?php


namespace Drupal\cri_demo\Form;

use Drupal;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;

class DemoForm extends FormBase
{

    /**
     * @inheritDoc
     */
    public function getFormId()
    {
        return 'cri_demo_ws_form';
    }

    /**
     * @inheritDoc
     */
    public function buildForm(array $form, FormStateInterface $form_state)
    {
      $currentdirectory = Drupal::service('extension.list.module')->getPath('cri_demo') . '/src/Resources';
      $modelJsonFile = $currentdirectory . '/demo.json';
      $json = Json::encode(Drupal::service('cri_core_mapping.service')->getSkeletoncontent('Request'));
      $triggerringElement = $form_state->getTriggeringElement();
      if (!empty($triggerringElement)) {
        switch ($triggerringElement['#value']) {
          case 'ws_multi_docs':
      $modelJsonFile = $currentdirectory . '/demo.json';
      $modelJsonFile = $currentdirectory . '/demo_multi_docs.json';
            break;
          case 'ws_mono_doc':
            $modelJsonFile = $currentdirectory . '/demo.json';
            break;
          default:
            $modelJsonFile = '';
            break;
        }
      }
      if (file_exists($modelJsonFile)) {
        $json = Json::encode(Json::decode(file_get_contents($modelJsonFile)));
      }
      // Modèle. Il est possible de recuperer un modèle par defaut.
      $modelFile = $currentdirectory . '/demo.docx';
      if (file_exists($modelFile)) {
        copy($modelFile, 'public://demo.docx');
        $file = File::create([
          'filename' => $this->t('Démonstration'),
          'uri' => 'public://demo.docx'
        ]);
        $file->save();
      }

      // Zip de la doc
      $docFileUri = Drupal::service('file_system')->realpath('public://documentation.zip');
      $sourceFile = Drupal::service('file_system')->realpath($currentdirectory . '/documentation_api.zip');
      copy($sourceFile, $docFileUri);

      $doc = File::create([
        'filename' => basename($docFileUri),
        'uri' => 'public://documentation.zip',
        'langcode' => 'fr'
      ]);
      $doc->save();

      $form['#attached']['library'][] = 'cri_demo/cri_jsoneditor';
      $form['form_wrapper'] = [
        '#type' => 'container',
        '#prefix' => '<div id="form-wrapper">',
        '#suffix' => '</div>',
        'ws_fields' => [
          'ws_request' => [
            '#type' => 'hidden',
            '#value' => $json,
          ]
        ],
        'ws_options' => [
          '#type' => 'content',
          '#prefix' => '<div class="row">',
          '#suffix' => '</div>',
          '#prefix' => '<h3>Espace démo - génération des documents</h3> | <a href="' . file_create_url($doc->getFileUri()) . '">' . $this->t('Guide d\'utilisation') . '</a>',
          'ws_remote_library' => [
            '#type' => 'checkbox',
            '#title' => $this->t('Tester la génération via un serveur distant (plus rapide)'),
            '#default_value' => TRUE,
            '#access' => true,
          ],
          'ws_mode_document' => [
            '#type' => 'select',
            '#title' => $this->t('Mode du flux'),
            '#options' => [
              'ws_mono_doc' => $this->t('Mono document'),
              'ws_multi_docs' => $this->t('Multi-documents'),
              'ws_free_input' => $this->t('Saisie libre')
            ],
            '#ajax' => [
              'callback' => [get_called_class(), 'refresh'],
            ]
          ],
          'ws_out_format' => [
            '#type' => 'select',
            '#title' => $this->t('Format de sortie'),
            '#options' => [
              'pdf' => $this->t('Document PDF'),
              'docx' => $this->t('Document Word'),
            ]
          ],
          'ws_model_file' => [
            '#type' => 'managed_file',
            '#title' => $this->t('Modèle [.docx]'),
            '#default_value' => $file ? [$file->id()] : NULL,
            '#upload_validators' => [
              'file_validate_extensions' => ['docx']
            ],
            '#accept' => '.docx',
          ],
          'jsoneditor-wrapper' => [
            '#type' => 'submit',
            '#ajax' => [
              'callback' => [get_called_class(),  'ajaxSubmit'],
              'wrapper' => 'form-wrapper'
            ],
            '#value' => $this->t('Exécuter')
          ]
        ],
        'ws_jsoneditor' => [
          '#prefix' => '<div class="row">',
          '#suffix' => '</div>',
          'jsoneditor_request' => [
            '#type' => 'markup',
            '#markup' => $this->t('Requête JSON'),
            '#prefix' => '<div class="layout-column layout-column--half jsoneditor--wrapper col-sm-12 col-md-6" data-input="ws_request">',
            '#suffix' => '</div>'
          ],
          'jsoneditor_response' => [
            '#type' => 'markup',
            '#markup' => $this->t('Réponse JSON'),
            '#prefix' => '<div class="layout-column layout-column--half jsoneditor--wrapper col-sm-12 col-md-6" data-input="ws_response">',
            '#suffix' => '</div>'
          ],
        ],
      ];

      return $form;
    }

    /**
     * @inheritDoc
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        // TODO: Implement submitForm() method.
    }

  /**
   * @param array $form
   * @param FormStateInterface $form_state
   * @return AjaxResponse
   */
    public static function ajaxSubmit(array &$form, FormStateInterface $form_state) {
      $inputs = $form_state->getUserInput();
      $data = Json::decode($inputs['ws_request']);
      $data['appel']['local'] = true;
      if (!empty($inputs['ws_model_file']['fids'])) {
        $fileLoadedId = $inputs['ws_model_file']['fids'];
        $file = File::load($fileLoadedId);
        foreach ($data['appel']['models'] as $key => $m) {
          $data['appel']['models'][$key]['format'] = $inputs['ws_out_format'];
          $data['appel']['models'][$key]['contentFile'] = base64_encode(file_get_contents($file->getFileUri()));
        }
      }


      $_remote = $inputs['ws_remote_library'] ?? FALSE;

      $result = Drupal::service('default.rest.editics.service')->_call($data, ['_type' => 'demo', '_remote' => $_remote]);
      $response = new AjaxResponse();
      $response->addCommand(new InvokeCommand(NULL, 'updateJsonResponse', ['ws_response', $result]));

      return $response;
    }

    public static function refresh(array $form, FormStateInterface $form_state) {
      $response = new AjaxResponse();
      return $response->addCommand(new ReplaceCommand('.cri-demo-ws-form', $form));
    }
}
